jQuery(function($){
    
    var xccsh={
        /** @var string selectedText Holds the text string selected in the editor to be wrapped in the shortcode. **/
        "selText" : null,
        /** @var textRange selTextRange Holds the selected TextRange object in IE needed for inserting the shortcode into the editor. The value is added when this.getSelection() is called. **/
        "selTextRange" : null,
        /** @var int selTextStart Holds the number of characters in the text area before the selection starts, needed for inserting the shortcode into the editor. For browsers other than IE. The value is added when this.getSelection() is called. **/
        "selTextStart" : null,
        /** @var int selTextEnd Holds the number of characters in the text area before the selection ends, needed for inserting the shortcode into the editor. For browsers other than IE. The value is added when this.getSelection() is called. **/
        "selTextEnd" : null,
        /** @var DOMElement canvas The editor DOMElement in which the text is selected. **/
        "canvas" : null,
        /** @var jQuery form The settings form in the pop up dialog. **/
        "form" : null,
        /**
         *Callback fired when the button is clicked
         *Gets the selected text then fires off an AJAX request for the options form
         *@param DOMElement element The button that was clicked
         *@param DOMElement canvas The editor DOMElement in which the text is selected
         *@param object ed The quicktags editor object
         */
        "qTagClick" : function(element, canvas, ed){
            this.canvas = canvas;
            //make sure we get the selected text before we create our pop-up dialog
            this.selText = xccsh.getSelection();
            //If the settings form dialog exists, show it
            if (this.form) {
                this.form.dialog('open');
            }
            //otherwise make an AJAX request to fetch it
            else{
                $.ajax(ajaxurl, {"data" : {"action" : "quicktags_dialog"}, "dataType" : "html", "success" : this.createOptionsDialog, "context": this});
            }
        },
        /**
         *Gets the selected text from the editor
         *See also qt.TagButton.prototype.callback in wp-includes/js/quicktags.js
         *@param DOMElement canvas The editor DOMElement in which the text is selected
         *@return string|null The selected text
         */
        "getSelection" : function(){
            var canvas = this.canvas;
            //IE - http://msdn.microsoft.com/en-us/library/ie/ms535869%28v=vs.85%29.aspx
            if (typeof document.selection != 'undefined'){
              canvas.focus();
              return (this.selTextRange = document.selection.createRange()).text;
            }
            //Everything else
            else if (typeof canvas.selectionStart != 'undefined'){
              return canvas.value.substring((this.selTextStart=canvas.selectionStart), (this.selTextEnd=canvas.selectionEnd));
            }
        },
        /**
         *Replaces the selected text in the editor with the supplied replacement
         *@param string replacement The text to use as the replacement for the currently selected text
         */
        "replaceSelection" : function(replacement){
            var canvas = this.canvas;
            canvas.focus();
            //IE
            if (this.selTextRange) {
                this.selTextRange.text=replacement;
                //move caret to end of selection
                this.selTextRange.collapse(false);
                this.selTextRange.select();
                canvas.focus();
            }
            //Other browsers
            else if(this.selTextStart !== null){
                var v = canvas.value,
                l = v.substring(0, this.selTextStart), //left of the selection
                r = v.substring(this.selTextEnd, v.length), //right of the selection
                cursorPos = l.length+replacement.length; //end of the selection when the replacement text has been inserted
                canvas.value = l + replacement + r;
                canvas.focus();
                //move the cursor after our inserted shortcode
                canvas.selectionStart = cursorPos;
                canvas.selectionEnd = cursorPos;
            }
        },
        /**
         *AJAX callback to create the settings form dialog
         *@param string data The settings form HTML
         */
        "createOptionsDialog" : function(data, textStatus, jqXHR){
            //create the optons form dialog
            var $data = this.form = $(data);
            //set the form in the object used for validating the settings, and hide the descriptions
            xccsh_shared.setForm($data).hideDescs();
            //create the dialog using jQuery UI
            $data.dialog({
                "height" : $(window).height()*0.8,
                "width" : 500,
                "title" : objectL10n.dialog_title,
                "buttons": {
                    /**
                     *Called when the button is clicked (form is 'submitted')
                     *Builds the shortcode from the form options, then inserts the shortcode into the editor (and closes the popup)
                     */
                    "Insert shortcode": function(e) {
                        if(!(replacement=xccsh_shared.getShortcode())){
                            return false;
                        }
                        xccsh.replaceSelection(replacement);
                        $( this ).dialog( "close" );
                    }
                },
                /**
                 *Resets the form when the dialog is closed
                 */
                "close" : function(e){
                    //remove zIndex hack from editor toolbar
                    $('#wp-content-editor-tools').css('z-index', '');
                    
                    //reset form values
                    var selTextEl=$('#xccsh_content');
                    selTextEl.val('');
                    $('#xccsh_from').val('');
                    $('#xccsh_to').val('');
                    $data.find('input[name="xccsh_options[static]"][value="default"]').prop('checked', true);
                    $('#xccsh_format').val('');
                    
                    //remove any error classes
                    xccsh_shared.remove_errors();
                    
                    xccsh.canvas.focus();
                },
                "open" : function(e){
                    $data.find('#xccsh_content').val(xccsh.selText).focus();
                    
                    //editor toolbar has a zIndex of 1000, which causes it to appear above the dialog
                    //set the toolbar to zIndex to 1 to avoid this problem
                    $('#wp-content-editor-tools').css('z-index', 1);
                }
            });
        }
    };
    
    //add the quicktag button
    QTags.addButton( 'xccsh', '$ => €', function(){return xccsh.qTagClick.apply(xccsh, arguments);}, null, null, 'add xoogu Currency Conversion shortcode' );
});